#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <compat/deprecated.h>
#include "ks0108.h"
#include <util/delay_basic.h>

//macros
#define BIT(num) (1<<num)
#define CSEL1() sbi(cmd_out,CS1);\
				cbi(cmd_out,CS2)
#define CSEL2() sbi(cmd_out,CS2);\
				cbi(cmd_out,CS1)
#define CSEL12() sbi(cmd_out,CS1);\
				sbi(cmd_out,CS2);
#define IS_CSEL1() (cmd_out&BIT(CS1))
#define IS_CSEL2() (cmd_out&BIT(CS2))

#define CEN() cbi(cmd_out,EN)
#define SEN() sbi(cmd_out,EN)
#define CDI() cbi(cmd_out,DI)
#define SDI() sbi(cmd_out,DI)
#define CRW() cbi(cmd_out,RW)
#define SRW() sbi(cmd_out,RW)

#define glcd_minDelay() _delay_loop_1(F_CPU/2000000)

#define CLOCK() SEN();\
	asm volatile("nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 "nop\n\t"\
				 ::);\
	CEN()
//
char invert_color = 0x0;

void glcd_init()
{
	cmd_dir |= BIT(EN)|BIT(RW)|BIT(DI)|BIT(CS1)|BIT(CS2);
	data_dir = 0xff;

	CSEL12();
	glcd_writeCommand(LCD_ON);
	glcd_writeCommand(LCD_DISP_START);
	glcd_clearScreen();
}

void glcd_writeCommand(unsigned char com)
{
	data_dir = 0xff;
	data_out = com;
	CDI();
	CRW();
	glcd_minDelay();
	CLOCK();
	glcd_minDelay();
}

void glcd_writedata(unsigned char data)
{
	data_dir = 0xff;
	data_out = data ^ invert_color;
	SDI();
	CRW();
	glcd_minDelay();
	CLOCK();
	glcd_minDelay();
}

char glcd_readdata()
{
	char t;
	data_dir = 0x00;
	data_out = 0xff;
	SRW();
	SDI();
	CLOCK();
	t = data_in;
	return t;
}

void glcd_clearScreen()
{
	char i , j;
	
	CSEL12();
	for(j = 0; j != 8 ; j++)
	{
		glcd_writeCommand(LCD_SET_PAGE | j);
		glcd_writeCommand(LCD_SET_ADD);
		i = 0;
		do
		{
			glcd_writedata(0);
			i++;
		}while(i != 64);
	}
}

void glcd_drawImage(unsigned char *ptr, unsigned char x , unsigned char y , unsigned char width , unsigned char high)
{
	char i , j;
	for( j = y; j != y+high; j++)
	{
		CSEL1();
		glcd_writeCommand(LCD_DISP_START);
		glcd_writeCommand(LCD_SET_ADD | (x&0x3f) );
		glcd_writeCommand(LCD_SET_PAGE | (j&0x07));
		
		for( i = x; i != x+width; i++ )
		{
			if( IS_CSEL1() )
				if( i >= 64 )
				{
					CSEL2();
					glcd_writeCommand(LCD_DISP_START);
					glcd_writeCommand(LCD_SET_ADD | (i&0x3f));
					glcd_writeCommand(LCD_SET_PAGE | (j&0x07));
				}
			
			
			glcd_writedata( *ptr );
			ptr++;
		}
	}
}

void glcd_drawImage_P(const uint8_t* ptr, unsigned char x , unsigned char y , unsigned char width , unsigned char high)
{
	char i , j;
	for( j = y; j != y+high; j++)
	{
		CSEL1();
		glcd_writeCommand(LCD_DISP_START);
		glcd_writeCommand(LCD_SET_ADD | (x&0x3f) );
		glcd_writeCommand(LCD_SET_PAGE | (j&0x07));
		
		for( i = x; i != x+width; i++ )
		{
			if( IS_CSEL1() )
				if( i >= 64 )
				{
					CSEL2();
					glcd_writeCommand(LCD_DISP_START);
					glcd_writeCommand(LCD_SET_ADD | (i&0x3f));
					glcd_writeCommand(LCD_SET_PAGE | (j&0x07));
				}
			
			
			glcd_writedata( pgm_read_byte(ptr) );
			ptr++;
		}
	}
}
